@@ -62,7 +62,10 @@ from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
 from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
-from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.AttributeNodes import (
+    ExpressionAttributeLookup,
+    StatementAssignmentAttribute
+)
 from nuitka.nodes.BuiltinFormatNodes import (
     ExpressionBuiltinAscii,
     ExpressionBuiltinFormat
@@ -84,7 +87,10 @@ from nuitka.nodes.ExceptionNodes import (
 )
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
-from nuitka.nodes.ModuleAttributeNodes import ExpressionModuleAttributeFileRef
+from nuitka.nodes.ModuleAttributeNodes import (
+    ExpressionModuleAttributeFileRef,
+    ExpressionModuleAttributeSpecRef
+)
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
     CompiledPythonPackage,
@@ -103,6 +109,7 @@ from nuitka.nodes.ReturnNodes import (
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
 from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
 from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
+from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
 from nuitka.Options import shallWarnUnusualCode
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
@@ -620,8 +627,11 @@ def buildConditionalExpressionNode(provider, node, source_ref):
 
 
 def buildAwaitNode(provider, node, source_ref):
-    return ExpressionAsyncWait(
-        expression = buildNode(provider, node.value, source_ref),
+    return ExpressionYieldFromWaitable(
+        expression = ExpressionAsyncWait(
+            expression = buildNode(provider, node.value, source_ref),
+            source_ref = source_ref
+        ),
         source_ref = source_ref
     )
 
@@ -662,11 +672,16 @@ def buildFormattedValueNode(provider, node, source_ref):
 
 
 def buildJoinedStrNode(provider, node, source_ref):
-    return ExpressionStringConcatenation(
-        values     = buildNodeList(provider, node.values, source_ref),
-        source_ref = source_ref
-    )
-
+    if node.values:
+        return ExpressionStringConcatenation(
+            values     = buildNodeList(provider, node.values, source_ref),
+            source_ref = source_ref
+        )
+    else:
+        return makeConstantRefNode(
+            constant   = "",
+            source_ref = source_ref
+        )
 
 
 setBuildingDispatchers(
@@ -809,7 +824,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                 provider      = provider,
                 variable_name = "__file__",
                 source        = ExpressionModuleAttributeFileRef(
-                    module     = provider,
+                    variable   = provider.getVariableForReference("__file__"),
                     source_ref = internal_source_ref,
                 ),
                 source_ref    = internal_source_ref
@@ -822,6 +837,31 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                 createPathAssignment(provider, internal_source_ref)
             )
 
+        if python_version >= 340 and not is_main:
+            statements += [
+                StatementAssignmentAttribute(
+                    source         = ExpressionModuleAttributeFileRef(
+                        variable   = provider.getVariableForReference("__file__"),
+                        source_ref = internal_source_ref,
+                    ),
+                    attribute_name = "origin",
+                    expression     = ExpressionModuleAttributeSpecRef(
+                        variable   = provider.getVariableForReference("__spec__"),
+                        source_ref = internal_source_ref,
+                    ),
+                    source_ref     = internal_source_ref,
+                ),
+                StatementAssignmentAttribute(
+                    source         = makeConstantRefNode(True, internal_source_ref),
+                    attribute_name = "has_location",
+                    expression     = ExpressionModuleAttributeSpecRef(
+                        variable   = provider.getVariableForReference("__spec__"),
+                        source_ref = internal_source_ref,
+                    ),
+                    source_ref     = internal_source_ref,
+                )
+            ]
+
     if python_version >= 300:
         statements.append(
             StatementAssignmentVariableName(
@@ -977,6 +1017,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
             result = CompiledPythonModule(
                 name         = module_name,
                 package_name = package,
+                is_top       = is_top,
                 mode         = Plugins.decideCompilation(full_name, source_ref),
                 future_spec  = None,
                 source_ref   = source_ref
@@ -984,15 +1025,17 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
 
     elif Importing.isPackageDir(filename):
         if is_top:
-            package_name = splitPath(filename)[-1]
+            module_name = splitPath(filename)[-1]
         else:
-            package_name = os.path.basename(filename)
+            module_name = os.path.basename(filename)
 
         source_filename = os.path.join(filename, "__init__.py")
 
         if not os.path.isfile(source_filename):
             source_ref, result = createNamespacePackage(
-                package_name   = package_name,
+                module_name    = module_name,
+                package_name   = package,
+                is_top         = is_top,
                 module_relpath = filename
             )
             source_filename = None
@@ -1002,13 +1045,14 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
             )
 
             if package is not None:
-                full_name = package + '.' + package_name
+                full_name = package + '.' + module_name
             else:
-                full_name = package_name
+                full_name = module_name
 
             result = CompiledPythonPackage(
-                name         = package_name,
+                name         = module_name,
                 package_name = package,
+                is_top       = is_top,
                 mode         = Plugins.decideCompilation(full_name, source_ref),
                 future_spec  = None,
                 source_ref   = source_ref
@@ -1024,9 +1068,6 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         )
         sys.exit(2)
 
-    if not Options.shallHaveStatementLines():
-        source_ref = source_ref.atInternal()
-
     return result, source_ref, source_filename
 
 
@@ -1039,8 +1080,6 @@ class CodeTooComplexCode(Exception):
         Example of this is "idnadata".
     """
 
-    pass
-
 
 def createModuleTree(module, source_ref, source_code, is_main):
     if Options.isShowMemory():
